//    SVM with Stoachstic Gradient Descent. Parallel implementation for Matlab.
//    Copyright (C) 2011 Marcus Rohrbach
//    Implementation of SGD based on code by Leon Bottou
//    see http://leon.bottou.org/projects/sgd.
//
//    This program is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    This program is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with this program.  If not, see <http://www.gnu.org/licenses/>.

#ifndef HELPERS_H_
#define HELPERS_H_

namespace {
inline
double loss(double z)
{
#if LOSS == LOGLOSS
	if (z > 18)
		return exp(-z);
	if (z < -18)
		return -z;
	return log(1+exp(-z));
#elif LOSS == LOGLOSSMARGIN
	if (z > 18)
		return exp(1-z);
	if (z < -18)
		return 1-z;
	return log(1+exp(1-z));
#elif LOSS == SMOOTHHINGELOSS
	if (z < 0)
		return 0.5 - z;
	if (z < 1)
		return 0.5 * (1-z) * (1-z);
	return 0;
#elif LOSS == SQUAREDHINGELOSS
	if (z < 1)
		return 0.5 * (1 - z) * (1 - z);
	return 0;
#elif LOSS == HINGELOSS
	if (z < 1)
		return 1 - z;
	return 0;
#else
# error "Undefined loss"
#endif
}

inline
double dloss(double z)
{
#if LOSS == LOGLOSS
	if (z > 18)
		return exp(-z);
	if (z < -18)
		return 1;
	return 1 / (exp(z) + 1);
#elif LOSS == LOGLOSSMARGIN
	if (z > 18)
		return exp(1-z);
	if (z < -18)
		return 1;
	return 1 / (exp(z-1) + 1);
#elif LOSS == SMOOTHHINGELOSS
	if (z < 0)
		return 1;
	if (z < 1)
		return 1-z;
	return 0;
#elif LOSS == SQUAREDHINGELOSS
	if (z < 1)
		return (1 - z);
	return 0;
#else
	if (z < 1)
		return 1;
	return 0;
#endif
}
}

inline double inner_product(const double x[], const floatType y[], size_t length) {
	double acc = 0.0;
	for (size_t i = 0; i < length; i++) {
		acc += x[i] * static_cast<double>(y[i]);
	}
	return acc;
}

template <class T> const T& max ( const T& a, const T& b ) {
	return (b<a)?a:b;
}


#endif /* HELPERS_H_ */
